@@ -6,6 +6,7 @@
 #if	ISO_CHAR_SET
 #include <locale.h>	/* support for 8bit character set:ew@senate.be */
 #endif
+#include <errno.h>
 
 extern char **environ;
 extern int errno;
@@ -18,6 +19,13 @@ extern FILE  *MESSAGEFILE;	/* file descriptor for important messages meant for t
 extern char  INDEX_DIR[MAX_LINE_LEN];
 extern struct stat istbuf;
 
+extern char	*TEMP_DIR; /* directory to store glimpse temporary files, usually /tmp unless -T is specified */
+#ifdef BUILDCAST
+/* TEMP_DIR is normally defined in ../main.c; if we're building
+ * buildcast, that's not linked in, so we need to define one here. */
+/* char * TEMP_DIR = NULL; */
+static char * TEMP_DIR = "/tmp";
+#endif /* BUILDCAST */
 extern int indexable_char[256];
 extern int GenerateHash;
 extern int KeepFilenames;
@@ -106,22 +114,23 @@ determine_sync()
 	int	i, ret;
 
 	strcpy(sync_path, "sync");
-	sprintf(S, "exec whereis sync > /tmp/zz.%d", getpid());
+	sprintf(S, "exec whereis sync > %s/zz.%d", TEMP_DIR,getpid());
 	/* Change it to use which: not urgent. */
 	system(S);
-	sprintf(S, "/tmp/zz.%d", getpid());
+	sprintf(S, "%s/zz.%d", TEMP_DIR,getpid());
 	if ((fp = fopen(S, "r")) == NULL) {
 		/* printf("11111\n"); */
 		return 0;
 	}
-	if ((ret = fread(S, 1, 1024, fp)) <= 0) {
-		sprintf(S, "/tmp/zz.%d", getpid());
+	if ((ret = fread(S, 1, sizeof(S)-1, fp)) <= 0) {
+		sprintf(S, "%s/zz.%d", TEMP_DIR,getpid());
 		unlink(S);
 		fclose(fp);
 		/* printf("22222\n"); */
 		return 0;
 	}
-	sprintf(s1, "/tmp/zz.%d", getpid());
+        S [ret] = 0; /* terminate string */
+	sprintf(s1, "%s/zz.%d", TEMP_DIR,getpid());
 	unlink(s1);
 	fclose(fp);
 	/* printf("read: %s\n", S); */
@@ -818,13 +827,13 @@ docleanup:
 
     if (AddToIndex || DeleteFromIndex || FastIndex) {
 	/* Not handling byte level indices here for now */
-	int	indextype, indexnumber, structuredindex, recordlevelindex, temp_attr_num, bytelevelindex;
+	int	indextype = 0, indexnumber = OFF, structuredindex = OFF, recordlevelindex = OFF, temp_attr_num = 0, bytelevelindex = OFF;
 	char	temp_rdelim[MAX_LINE_LEN];
 
 	sprintf(s, "%s/%s", INDEX_DIR, INDEX_FILE);
 	if (-1 == stat(s, &istbuf)) {
 	    if (AddToIndex || DeleteFromIndex) {
-		fprintf(stderr, "Cannot find previous index! Fresh indexing recommended\n", s);
+		fprintf(stderr, "Cannot find previous index %s! Fresh indexing recommended\n", s);
 		return usage(0);
 	    }
 	    file_num = 0;
@@ -1033,9 +1042,9 @@ docleanup:
 
     printf("\nIndex-directory: \"%s\"\nGlimpse-files created here:\n", INDEX_DIR);
     chdir(INDEX_DIR);
-    sprintf(s, "exec %s -l .glimpse_* > /tmp/%d\n", SYSTEM_LS, pid);
+    sprintf(s, "exec %s -l .glimpse_* > %s/%d\n", SYSTEM_LS, TEMP_DIR,pid);
     system(s);
-    sprintf(s, "/tmp/%d", pid);
+    sprintf(s, "%s/%d", TEMP_DIR,pid);
     if ((tmpfp = fopen(s, "r")) != NULL) {
 	memset(tmpbuf, '\0', 1024);
 	while(fgets(tmpbuf, 1024, tmpfp) != NULL) fputs(tmpbuf, stdout);
