@@ -1,9 +1,11 @@
 /* Copyright (c) 1994 Sun Wu, Udi Manber, Burra Gopal.  All Rights Reserved. */
 /* ./glimpse/index/io.c */
 #include "glimpse.h"
+#include <autoconf.h>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <dlfcn.h>
 extern char INDEX_DIR[MAX_LINE_LEN];
 extern int memory_usage;
 
@@ -151,7 +153,6 @@ int  overflow_ok;
 		if ((c = getc(file_in)) == EOF) break;
 		val |= c;
 
-		table[tx++] = val;
 		if(tx > max_entry) {
 			if(!overflow_ok) {
 			    fprintf(stderr, "in get_table: table overflow\n");
@@ -159,6 +160,7 @@ int  overflow_ok;
 			}
 			break;
 		}
+		table[tx++] = val;
 	}
 	fclose(file_in);
 	return(tx);
@@ -221,6 +223,149 @@ CHAR *filter[MAX_FILTER];
 CHAR *filter_command[MAX_FILTER];
 struct stat filstbuf;
 
+
+
+/* Prototype for filter entry point in a shared library -- CV 9/14/99 */
+typedef int (*FILTER_FUNC)(FILE *, FILE *);
+
+/* Holds addresses of entry points -- CV 9/14/99 */
+static FILTER_FUNC filter_func[MAX_FILTER];
+
+/* Holds shared library handles, one per filter and shared library 
+   -- CV 9/14/99 */
+static void *filter_handle[MAX_FILTER];
+
+
+/* Loads shared filter libraries. The criterion, upon which this
+   function decides whether a filter is an external program or a shared
+   library is as follows: If the name of the filter command ends in ".so",
+   it assumes that the filter is a shared library; otherwise it assumes
+   an external program.
+ 
+   apply_filters() finds out what kind the ith filter is used by
+   looking at filter_func[i]. If it is non-null, it is a pointer to
+   the entry point in a shared library. Otherwise, the filter is an
+   external program. -- CV 9/14/99
+*/
+
+#ifndef	RTLD_NOW
+/* dummy function for old system such as SunOS-4.1.3 */
+static void load_dyn_filters(void) {}
+#else
+static void load_dyn_filters(void)
+{
+    int i, len, success;
+    char *so_pos;
+    char *error;
+
+    memset(filter_func, '\0', sizeof (filter_func));
+    memset(filter_handle, '\0', sizeof (filter_handle));
+
+    for (i = 0; i < num_filter; i++) {
+	success = 1;
+	len = strlen(filter_command[i]);
+	if (len > 4) {
+	    /* find location in string where .so suffix should be */
+	    so_pos = filter_command[i] + len - 3;
+	    if (strcmp(so_pos, ".so") == 0) {
+		/* fprintf(stderr, "Loading %s\n", filter_command[i]); */
+		if ((filter_handle[i] = dlopen(filter_command[i],
+					       RTLD_NOW)) == NULL) {
+		    success = 0;
+		    error = dlerror();
+		}
+		if (success) {
+		    filter_func[i] = dlsym(filter_handle[i], "filter_func");
+		    if ((error = dlerror()) != NULL)
+			success = 0;
+		}
+		if (! success) {
+		    fputs("Warning: Error while loading filter\n", stderr);
+		    fputs(error, stderr); 
+		    if (filter_handle[i] != NULL) {
+                       /* lib was already loaded  when error happened */
+			dlclose(filter_handle[i]);
+		    } 
+		    /* Since an error occurred, should we disable the
+                       filter entirely for this command?  If yes, HOW? -- CV */
+		}
+	    }
+	}
+    }
+}
+#endif	/* RTLD_NOW */
+
+
+/* Applies a filter to a single file, based on whether the filter is a
+   command or in a shared library.
+
+   filter_number is the filter's index in the filter table.
+   Returns 0 on success, passes on error code from the filter otherwise
+   -- CV 9/14/99
+ */
+
+static int apply_one_filter(int filter_number, const char *in_name,
+		     const char *out_name)
+{
+    int ret = 0;
+
+    if (filter_func[filter_number] != NULL) { /* in shared library */
+	FILE *in = NULL, *out = NULL;
+	in = fopen(in_name, "r");
+	if (in != NULL) out = fopen(out_name, "w"); 
+
+	if (in == NULL || out == NULL) ret = errno;
+	else ret = (*filter_func[filter_number])(in, out);
+
+	if (out != NULL) fclose(out);
+	if (in != NULL) fclose(in);
+    } 
+    else { /* in external program */
+	char escaped_in[MAX_LINE_LEN], escaped_out[MAX_LINE_LEN];	
+	char command[2 * MAX_LINE_LEN];
+	escapesinglequote(in_name, escaped_in);
+	escapesinglequote(out_name, escaped_out);
+	sprintf(command, "exec %s '%s' > '%s'", filter_command[filter_number],
+		escaped_in, escaped_out);
+	/* FIXME: use snprintf() where available to avoid stupid
+	   buffer overruns. But security risk should be low, because
+	   no user-supplied data goes in here. -- CV 9/14/99. */
+
+	ret = system(command);
+    }
+    return ret;
+}
+
+
+/* Copies a file verbatim. It could be a little better optimized by
+   using fread() and fwrite(), but speed is not critical here.
+   Returns 0 on success, error code otherwise.
+   -- CV 9/14/99
+*/
+
+static int copy_file(const char *source, const char *destination) {
+    FILE *src = NULL, *dest = NULL;
+    int c, ret = 0;
+
+    src = fopen(source, "r");
+    if (src != NULL) dest = fopen(destination, "w");
+    if (src == NULL || dest == NULL) 
+	ret = errno;
+    else {
+	/* FIXME: Better error checking here. But it's not really critical,
+	   because these are only temporary files.  -- CV 9/14/99 */
+	while ((c = fgetc(src)) != EOF)
+	    fputc(c, dest);
+    }
+
+    if (dest != NULL) fclose(dest);
+    if (src != NULL) fclose(src);
+    return ret;
+}
+
+
+
+
 read_filters(index_dir, dofilter)
 char	*index_dir;
 int	dofilter;
@@ -295,6 +440,8 @@ int	dofilter;
 	}
 	fclose(filterfile);
     }
+
+    load_dyn_filters(); /* load filters in shared libraries -- CV 9/14/99 */
 }
 
 /* 1 if filter application was successful and the output (>1B) is in outname, 2 if some pattern matched but there is no output, 0 otherwise: sep 15-18 '94 */
@@ -335,8 +482,9 @@ apply_filter(inname, outname)
 				if (ret == 0) ret = 2;
 				/* yes, it matched: now apply the command and get the output */
 				/* printf("filtering %s\n", name); */
-				sprintf(s, "exec %s '%s' > '%s'", filter_command[i], escapesinglequote(prevoutname, es1), escapesinglequote(newoutname, es2));
-				system(s);
+				/* new filter function -- CV 9/14/99 */
+				apply_one_filter(i, prevoutname, newoutname);
+
 				if (((dummyin = my_fopen(newoutname, "r")) == NULL) || (fread(dummybuf, 1, 1, dummyin) <= 0)) {
 					if (dummyin != NULL) fclose(dummyin);
 					unlink(newoutname);
@@ -379,8 +527,9 @@ apply_filter(inname, outname)
 				if (ret == 0) ret = 2;
 				/* yes, it matched: now apply the command and get the output */
 				/* printf("filtering %s\n", name); */
-				sprintf(s, "exec %s '%s' > '%s'", filter_command[i], escapesinglequote(prevoutname, es1), escapesinglequote(newoutname, es2));
-				system(s);
+				/* new filter function -- CV 9/14/99 */
+ 				apply_one_filter(i, prevoutname, newoutname);
+
 				if (((dummyin = my_fopen(newoutname, "r")) == NULL) || (fread(dummybuf, 1, 1, dummyin) <= 0)) {
 					if (dummyin != NULL) fclose(dummyin);
 					unlink(newoutname);
@@ -406,8 +555,8 @@ apply_filter(inname, outname)
 	}
 	if (ret == 1) strcpy(outname, prevoutname);
 	else {	/* dummy filter that copies input to output: caller can use tempinname but this has easy interface */
-		sprintf(s, "exec %s '%s' > '%s'\n", SYSTEM_CAT, escapesinglequote(tempinname, es1), escapesinglequote(outname, es2));
-		system(s);
+	    /* replaced system() call with a simple copy function. -- CV 9/14/99 */
+	    copy_file(tempinname, outname);
 	}
 	fclose(dummyout);
 	return ret;
@@ -759,6 +908,8 @@ dump_mini(indexfile)
 	long	offset;	/* offset if offset of beginning of word */
 	char	temp_rdelim[MAX_LINE_LEN];
 
+	temp_rdelim[0] = '\0';  /* Initialize just in case. 10/25/99 --GV */
+
 	if ((indexfp = fopen(indexfile, "r")) == NULL) {
 		fprintf(stderr, "Can't open for reading: %s\n", indexfile);
 		return;
@@ -909,6 +1060,8 @@ dump_mini(indexfile)
 	int	rc;	/* return code from system(3) */
 	char	es1[MAX_LINE_LEN], es2[MAX_LINE_LEN], es3[MAX_LINE_LEN], temp_rdelim[MAX_LINE_LEN];
 
+	temp_rdelim[0] = '\0';  /* Initialize in case not read. 10/25/99 --GV */
+
 	/*
 	 * First change the sorting order of the index file.
 	 */
@@ -927,7 +1080,8 @@ dump_mini(indexfile)
 	/* Must store since sort -n can screw it up */
 	fgets(indexnumber, 256, indexfp);
 	fgets(onefileperblock, 256, indexfp);
-	fscanf(indexfp, "%%%d%s\n", &attr_num, temp_rdelim);
+	if ( !fscanf(indexfp, "%%%d%s\n", &attr_num, temp_rdelim)) 
+		fscanf(indexfp, "%%%d\n", &attr_num);
 
 	while (fgets(s, MAX_LINE_LEN, indexfp) != NULL) {
 		j = 0;
@@ -1085,19 +1239,23 @@ int	files;
 	FILEMASK_SIZE = ((files + 1)/(8*sizeof(int)) + 4);
 	REAL_PARTITION = (FILEMASK_SIZE + 4);
 	if (REAL_PARTITION < MAX_PARTITION + 2) REAL_PARTITION = MAX_PARTITION + 2;
-	REAL_INDEX_BUF = ((files + 1)  + 2*MAX_WORD_BUF + 2);	/* index line length with OneFilePerBlock (and/or ByteLevelIndex) */
+	REAL_INDEX_BUF = ((files + 1)*3 + 3*8*sizeof(int)  + 6*MAX_WORD_BUF + 2);	/* index line length with OneFilePerBlock (and/or ByteLevelIndex) */
+	/* increased by *3 + 24*sizeof(int) to avoid segfaults --GV */
 	if (REAL_INDEX_BUF < MAX_SORTLINE_LEN) REAL_INDEX_BUF = MAX_SORTLINE_LEN;
 	MAX_ALL_INDEX = (REAL_INDEX_BUF / 2);
 	if (src_index_set == NULL) src_index_set = (unsigned int *)my_malloc(sizeof(int)*REAL_PARTITION);
 	memset(src_index_set, '\0', sizeof(int) * REAL_PARTITION);
 	if (dest_index_set == NULL) dest_index_set = (unsigned int *)my_malloc(sizeof(int)*REAL_PARTITION);
 	memset(dest_index_set, '\0', sizeof(int) * REAL_PARTITION);
-	if (src_index_buf == NULL) src_index_buf = (unsigned char *)my_malloc(sizeof(char)*REAL_INDEX_BUF);
-	memset(src_index_buf, '\0', sizeof(char)*REAL_INDEX_BUF);
-	if (dest_index_buf == NULL) dest_index_buf = (unsigned char *)my_malloc(sizeof(char)*REAL_INDEX_BUF);
-	memset(dest_index_buf, '\0', sizeof(char)*REAL_INDEX_BUF);
-	if (merge_index_buf == NULL) merge_index_buf = (unsigned char *)my_malloc(sizeof(char)*REAL_INDEX_BUF);
-	memset(merge_index_buf, '\0', sizeof(char)*REAL_INDEX_BUF);
+
+/* malloc a few extra bytes above REAL_INDEX_BUF */
+/* the value of REAL_INDEX_BUF is used to terminate loops, but in the loop up to 4 chars may be assigned */
+	if (src_index_buf == NULL) src_index_buf = (unsigned char *)my_malloc(sizeof(char)*(REAL_INDEX_BUF+4));
+	memset(src_index_buf, '\0', sizeof(char)*(REAL_INDEX_BUF+4));
+	if (dest_index_buf == NULL) dest_index_buf = (unsigned char *)my_malloc(sizeof(char)*(REAL_INDEX_BUF + 4));
+	memset(dest_index_buf, '\0', sizeof(char)*(REAL_INDEX_BUF+4));
+	if (merge_index_buf == NULL) merge_index_buf = (unsigned char *)my_malloc(sizeof(char)*(REAL_INDEX_BUF + 4));
+	memset(merge_index_buf, '\0', sizeof(char)*(REAL_INDEX_BUF+4));
 }
 
 destroy_data_structures()
@@ -1131,9 +1289,10 @@ special_parse_name(name, len, begin, end)
 	*begin = -1;
 	*end = -1;
 	if (InfoAfterFilename || ExtractInfo) {	/* Glimpse will ALWAYS terminate filename at first blank (no ' ', '\n', '\0' in filename) */
+		/* Trying to use FILE_END_MARK instead of blank! --GB 6/7/99 */
 		for (i=0; i<len; i++) {
 			if (name[i] == '\n') break;
-			if (name[i] == ' ')  {
+			if (name[i] == FILE_END_MARK)  {
 				if (*begin == -1) {
 					*begin = i+1;
 					if (!InfoAfterFilename) break;	/* don't care about URL since it doesn't exist as far as I know */
@@ -1252,6 +1411,7 @@ my_lstat(name, buf)
 
 /* Changed hash-routines to look at exactly that portion of the filename that occurs before the first blank character, */
 /* and use that to compare names: Oct/96 --- But lose efficiency since must parse name everytime: at least 1 string copy */
+/* Using FILE_END_MARK instead of blank. --GB 6/7/99 */
 name_hashelement *name_hashtable[MAX_64K_HASH];	/* if (!BigFilenameHashTable) then only the first 4K entries in it are used */
 
 /*
