@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: template.c,v 1.17 1995/02/01 22:12:59 hardy Exp $";
+static char rcsid[] = "$Id: template.c,v 1.2 2003/11/13 05:17:35 golda Exp $";
 /*
  *  template.c - SOIF Object ("template") processing code for Harvest 
  *
@@ -592,7 +592,7 @@ Template *parse_template()
 	template->list = NULL;
 
 #ifdef DEBUG
-	log("Grabbing Template Object: %s %s\n", template->template_type,
+	glimpselog("Grabbing Template Object: %s %s\n", template->template_type,
 	    template->url);
 #endif
 
